/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.service.utils

import org.slf4j.LoggerFactory
import javax.imageio.ImageIO
import javax.servlet.ServletOutputStream
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import java.awt.*
import java.awt.image.BufferedImage
import java.io.IOException
import java.util.Random

/**
 * 验证码生成器
 *
 * @author 庞利祥 <sybotan@126.com>
 */
object SRandCodeUtil {
    /** 日志 */
    private val logger = LoggerFactory.getLogger(SRandCodeUtil::class.java)

    /**
     * 生成验证码
     *
     * @param request  客户端请求对象
     * @param response 服务器应答对象
     */
    fun checkCode(request: HttpServletRequest, response: HttpServletResponse) {
        val width = 195         /** 验证码图片的宽度 */
        val height = 26         /** 验证码图片的高度 */
        val codeCount = 4       /** 验证码字符个 */
        val xx: Int
        val fontHeight: Int     /** 字体高度 */
        val codeY: Int
        val codeSequence = charArrayOf('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '1', '2', '3', '4', '5', '6', '7', '8', '9')
        xx = width / (codeCount + 1)
        fontHeight = height - 2
        codeY = height - 5

        /** 定义图像buffer */
        val buffImg = BufferedImage(width, height, BufferedImage.TYPE_INT_RGB)
        val gd = buffImg.createGraphics()

        /** 创建随机数生成器 */
        val random = Random()

        /** 将图像填充为白色 */
        gd.color = Color.WHITE
        gd.fillRect(0, 0, width, height)

        /** 创建字体，字体的大小应该根据图片的高度来定 */
        /** 字体 */
        val fontTypes = arrayOf("tahoma", "Atlantic Inline", "fantasy", "Times New Roman", "Georgia", "Arial", "Helvetica", "sans-serif", "System")
        val fontTypesLength = fontTypes.size
        val font = Font(fontTypes[random.nextInt(fontTypesLength)], Font.BOLD, fontHeight)
        /** 设置字体 */
        gd.font = font

        gd.color = Color.red
        for (i in 0..99) {
            val x = random.nextInt(width)
            val y = random.nextInt(height)
            gd.drawOval(x, y, 0, 0)
        }

        /** randomCode用于保存随机产生的验证码，以便用户登录后进行验证 */
        val randomCode = StringBuffer()
        var red: Int
        var green: Int
        var blue: Int

        /** 随机产生codeCount数字的验证码 */
        for (i in 0 until codeCount) {
            /** 得到随机产生的验证码数字 */
            val strRand = codeSequence[random.nextInt(34)].toString()
            /** 产生随机的颜色分量来构建颜色值，这样输出的每位数字的颜色值都将不同 */
            red = random.nextInt(200)
            green = random.nextInt(200)
            blue = random.nextInt(200)

            /** 用随机产生的颜色将验证码绘制到图像中 */
            gd.color = Color(red, green, blue)
            gd.drawString(strRand, (i + 1) * xx, codeY)

            /** 将产生的四个随机数组合在 */
            randomCode.append(strRand)
        }

        /** 将四位数字的验证码保存到 Session 中 */
        logger.debug("verifyCode = $randomCode")
        val session = request.session
        session.setAttribute("verifyCode", randomCode.toString())

        /** 禁止图像缓存 */
        response.setHeader("Pragma", "no-cache")
        response.setHeader("Cache-Control", "no-cache")
        response.setDateHeader("Expires", 0)
        response.contentType = "image/jpeg"

        /** 将图像输出到Servlet输出流中 */
        val sos: ServletOutputStream
        try {
            sos = response.outputStream
            ImageIO.write(buffImg, "jpeg", sos)
            sos.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
        return
    }

}
