/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.service.utils.crypt

import javax.crypto.Cipher
import javax.crypto.KeyGenerator
import javax.crypto.spec.SecretKeySpec
import java.io.IOException
import java.security.Key
import java.security.NoSuchAlgorithmException
import java.security.SecureRandom
import java.util.*

/**
 * AES 安全编码组件
 *
 * @author Andy
 */
abstract class SAESCoderUtil : SCoderUtil() {
    companion object {
        const val ALGORITHM = "AES"

        /**
         * 转换密钥
         *
         * @param key 字节数组
         * @return key
         * @throws Exception
         */
        @Throws(Exception::class)
        private fun toKey(key: ByteArray): Key {
            return SecretKeySpec(key, ALGORITHM)
        }

        /**
         * 解密
         *
         * @param data
         * @param key
         * @return
         * @throws Exception
         */
        @Throws(Exception::class)
        fun decrypt(data: ByteArray, key: String): ByteArray {
            val k = toKey(Base64.getDecoder().decode(key))

            val cipher = Cipher.getInstance(ALGORITHM)
            cipher.init(Cipher.DECRYPT_MODE, k)

            return cipher.doFinal(data)
        } // Fun decrypt()

        /**
         * 加密
         *
         * @param data
         * @param key
         * @return
         * @throws Exception
         */
        @Throws(Exception::class)
        fun encrypt(data: ByteArray, key: String): ByteArray {
            val k = toKey(Base64.getDecoder().decode(key))
            val cipher = Cipher.getInstance(ALGORITHM)
            cipher.init(Cipher.ENCRYPT_MODE, k)

            return cipher.doFinal(data)
        } // Fun encrypt()

        /**
         * 生成密钥
         *
         * @param seed
         * @return
         * @throws IOException
         * @throws NoSuchAlgorithmException
         */
        @Throws(IOException::class, NoSuchAlgorithmException::class)
        @JvmOverloads
        fun initKey(seed: String? = null): String? {
            val secureRandom = if (seed != null) {
                SecureRandom(Base64.getDecoder().decode(seed))
            } else {
                SecureRandom()
            }

            val kg = KeyGenerator.getInstance(ALGORITHM)
            kg.init(secureRandom)

            val secretKey = kg.generateKey()

            return Base64.getEncoder().encodeToString(secretKey.encoded)
        }
    }
}

