/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.service.models.requests

import com.persagy.database.SGroup
import com.persagy.service.models.SCascadeQuery
import io.swagger.v3.oas.annotations.media.Schema


/**
 * 查询请求对象基类
 *
 * @author 庞利祥 <sybotan@126.com>
 */
@Schema(description = "查询请求对象基类")
open class SQueryRequest {
    companion object {
        /** 设置接口一次可取得的最大记录数 */
        var maxRow = 1000
    }

    /** 第几页 */
    @Schema(description = "第几页(从1开始)", example = "1")
    var pageNumber: Int = 1
        set(value) {            // 限定pageNumber不能小于1
            field = value
            if (value < 1) {
                field = 1
            }
        }

    /** 每页记录数 */
    @Schema(description = "每页记录数(0 < 记录数 < 1000)", example = "50")
    var pageSize: Int = 50
        set(value) {            // 限定pageSize不能小于1且不能大于maxRow
            field = value
            if (value < 1 ) {
                field = 1
            }
            if (value > maxRow) {
                field = maxRow
            }
            // maxRow一次设置有效，之后将自动设备为1000
            maxRow = 1000
        }

    /** 查询条件 */
    @Schema(description = "查询条件")
    var filters: String? = null

    /** 排序条件 */
    @Schema(description = "排序条件", example = "name asc, createTime desc")
    var orders: String? = null

    /** 去除重复记录 */
    @Schema(description = "去除重复记录")
    var distinct: Boolean? = null

    /** 字段影射，空为查询所有字段 */
    @Schema(description = "字段影射，空为查询所有字段")
    var projection: ArrayList<String>? = null

    /** 级联对象列表 */
    @Schema(description = "级联对象列表")
    var cascade: ArrayList<SCascadeQuery>? = null

    /** 分组统计 */
    @Schema(description = "分组")
    var group: SGroup? = null
}