/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.service.mysql.controllers

import com.persagy.service.SObjectService
import com.persagy.service.mysql.models.SColumn
import com.persagy.service.mysql.models.STable
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.web.bind.annotation.*

/**
 * 数据库设计接口
 *
 * @author 庞利祥 <sybotan@126.com>
 */
@Tag(name = "数据库设计接口")
@RestController
@RequestMapping("/--database-doc--/")
class SDatabaseDocController {
    /** 表服务对象 */
    private val tableService = SObjectService(STable::class.java)

    /** 字段服务对象 */
    private val columnService = SObjectService(SColumn::class.java)

    /**
     * 查询表信息
     *
     * @param schema      数据库
     * @return 字段列表
     */
    @Operation(summary="查询表信息",description = "")
    @RequestMapping(value = ["/table-list"], method = [RequestMethod.GET, RequestMethod.POST])
    fun tableList(schema: String): List<Any> {
        return tableService.select("schemaName" to schema).exec()
    }

    /**
     * 查询字段信息
     *
     * @param schema      数据库
     * @param table       表名
     * @return 字段列表
     */
    @Operation(summary="查询字段信息", description="")
    @RequestMapping(value = ["/column-list"], method = [RequestMethod.GET, RequestMethod.POST])
    fun columnList(schema: String, table: String): ArrayList<SColumn> {
        val builder = columnService.select("schemaName" to schema, "tableName" to table)
        builder.tableName =
                "(SELECT *,\n" +
                "    (SELECT \"description\" FROM \"pg_description\"\n" +
                "    JOIN \"pg_class\" ON \"pg_description\".\"objoid\" = \"pg_class\".\"oid\"\n" +
                "    JOIN \"pg_namespace\" ON \"pg_class\".\"relnamespace\" = \"pg_namespace\".\"oid\"\n" +
                "WHERE \"relname\" = \"t\".\"table_name\" and \"nspname\"= \"t\".\"table_schema\" and \"objsubid\" = \"t\".\"ordinal_position\")\n" +
                "FROM \"information_schema\".\"columns\" t) ct"
        return builder.exec()
        //return mapper.columnList(schema, table)
    }


    /**
     * 查询表、字段信息
     *
     * @param schema    字段
     * @param table     表名
     * @return
     */
    @Operation(summary="查询字段信息", description="")
    @RequestMapping(value = ["/table-info"], method = [RequestMethod.GET, RequestMethod.POST])
    fun tableInfo(schema: String, table: String): STable {
        val sTable = STable()
        sTable.schemaName = schema
        sTable.tableName = table
        sTable.columnList = columnList(schema, table)
        return sTable
    }

}